<?php
/**
 * USMS base class
 *
 * Copyright 2007 Obala d.o.o. (www.obala.si)
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://cvs.horde.org/co.php/news/LICENSE.
 *
 * $Id: Driver.php 310 2008-01-31 15:52:22Z duck $
 *
 * @author Duck <duck@obala.net>
 * @package Usms
 */
class Usms_Driver {

    /**
     * Hash containing connection parameters.
     *
     * @var array
     */
    private $_params = array();

    /**
     * Handle for the current database connection.
     *
     * @var DB
     */
    private $_db;

    /**
     * Handle for the current database connection, used for writing. Defaults
     * to the same handle as $_db if a separate write database is not required.
     *
     * @var DB
     */
    private $_write_db;

    /**
     * Cache object
     *
     * @var Horde_Cache
     */
    private $_cache;

    /**
     * Constructor
     */
    public function __construct($params)
    {
        /* Connect to DB. */
        $this->_params = Horde::getDriverConfig('storage', 'sql');
        $this->_connect();

        /* Initialize the Cache object. */
        $this->_cache = &Horde_Cache::singleton($GLOBALS['conf']['cache']['driver'],
                                                Horde::getDriverConfig('cache', $GLOBALS['conf']['cache']['driver']));
    }

    /**
     * Singleton
     */
    public function &singleton($driver = 'unixfs', $params = array())
    {
        static $usms = array();

        $signature = serialize($params);
        if (!isset($usms[$signature])) {

            $class_name = 'Usms_Driver_' . $driver;
            require_once USMS_BASE . '/lib/Driver/' . $driver . '.php';
            if (!class_exists($class_name)) {
                Horde::fatal('DRIVER MISSING', __FILE__, __LINE__);
            }

            $usms[$signature] = new $class_name($params);
        }

        return $usms[$signature];
    }

    /**
     * Get message
     *
     * @param integer $id Message id
     * @param string $folder Message folder
     */
    public function getMessage($id, $folder = 'inbox')
    {
        $query = 'SELECT status, user_to, user_from AS user, priority, title, chars, attaches FROM usms_';
        if ($folder == 'sent') {
            $query .= 'inbox WHERE id=? AND user_from=? ORDER BY id';
        } else {
            $query .= $folder . ' WHERE id=? AND user_to=? ORDER BY id';
        }

        $row = $this->_db->getRow($query, array($id, Auth::getAuth()), DB_FETCHMODE_ASSOC);
        if ($row instanceof PEAR_Error) {
            return $row;
        } elseif (empty($row)) {
            return PEAR::RaiseError(_("There message your requested dosent exits."));
        }

        $data = $this->readBodyData($id, $row['user_to']);
        if ($data instanceof PEAR_Error) {
            return $data;
        }

        $row = array_merge($row, $data);
        unset($data);

        require_once 'Horde/Text/Filter.php';
        $row['content'] = Text_Filter::filter($row['content'],
                                            array('text2html', 'emoticons', 'highlightquotes', 'emails', 'linkurls'),
                                            array(array('parselevel' => TEXT_HTML_NOHTML),
                                                    array('entities' => true),
                                                    array('citeblock' => true),
                                                    array('always_mailto' => true),
                                                    array('targer' => '_blank')));

        if ($row['status'] == Usms::STATUS_UNSEEN && $row['user_to'] == Auth::getAuth()) {
            $this->changeStatus($id, $folder, Usms::STATUS_SEEN);

            if (!empty($row['notify'])) {
                $message = array('title' => _("Message was read"),
                                 'content' => sprintf(_("Your message \"%s\" to %s was read."),
                                                        $row['title'], $row['user_to']));
                $this->sendMessage($row['user'], $message, false);
            }
        }

        return $row;
    }

    /**
     * Get user messages
     *
     * @param array $criteria Filter criteria
     * @param integer $from Offset
     * @param integer $count Count
     */
    public function getMessages($criteria = array(), $from = 0, $count = 0)
    {
        $key = md5(serialize($criteria) . $from . $count);
        $msgs = $this->_getCache($key);
        if ($msgs) {
            return unserialize($msgs);
        }

        $binds = $this->_buildWhere($criteria, false);
        if (!isset($criteria['sort_by'])) {
            $criteria['sort_by'] = $GLOBALS['prefs']->getValue('sortby');
        }
        if (!isset($criteria['sort_dir'])) {
            $criteria['sort_dir'] = $GLOBALS['prefs']->getValue('sortdir');
        }

        $user = ($criteria['folder'] == 'sent') ? 'user_to' : 'user_from';

        $binds[0] = 'SELECT id, status, ' . $user . '  AS user, priority, title, chars, attaches ' . $binds[0]
                    . ' ORDER BY ' . $criteria['sort_by']
                    . ' ' . $criteria['sort_dir'];

        if ($count) {
            $binds[0] = $this->_db->modifyLimitQuery($binds[0], $from, $count);
        }

        $msgs = $this->_db->getAll($binds[0], $binds[1], DB_FETCHMODE_ASSOC);
        if (!($msgs instanceof PEAR_Error)) {
            $this->_setCache($key, serialize($msgs));
        }

        return $msgs;
    }

    /**
     * Count user messages
     *
     * @param array $criteria Filter criteria
     * @param integer $from Offset
     */
    public function countMessages($criteria)
    {
        $key = md5(serialize($criteria) . '_count');
        $msgs = $this->_getCache($key);
        if ($msgs) {
            return $msgs;
        }

        $binds = $this->_buildWhere($criteria);
        $binds[0] = 'SELECT COUNT(*) ' . $binds[0];

        $msgs = $this->_db->getOne($binds[0], $binds[1]);
        if (!($msgs instanceof PEAR_Error)) {
            $this->_setCache($key, $msgs);
        }

        return $msgs;
    }

    /**
     * Prepare query
     *
     * @param array $criteria Filter criteria
     */
    private function _buildWhere($criteria)
    {
        if ($criteria['folder'] == 'sent') {
            $query = ' FROM usms_inbox WHERE user_from = ? ';
        } else {
            $query = ' FROM usms_' . $criteria['folder'] . ' WHERE user_to = ? ';
        }

        $params = array();
        $params[] = ($criteria['user']) ? $criteria['user'] : Auth::getAuth();

        if ($criteria['hide_deleted']) {
            $query .= ' AND status <> ? ';
            $params[] = Usms::STATUS_DELETED;
        }

        return array($query, $params);
    }

    /**
     * Check message(s) status
     *
     * @param integer $id Message id
     * @param string $folder Message folder
     * @param integer $status Message status
     */
    public function changeStatus($id, $folder = 'inbox', $status = 0)
    {
        if (is_array($id)) {
            $query = 'UPDATE usms_' . $folder . ' SET status="' . (int)$status
                    . '" WHERE id IN (' . implode(',', $id) . ') AND user_to="'
                    . Auth::getAuth() .  '"';
        } else {
            $query = 'UPDATE usms_' . $folder . ' SET status="' . (int)$status
                    . '" WHERE id  = "' . $id . '" AND user_to="'
                    . Auth::getAuth() .  '"';
        }

        $result = $this->_write_db->query($query);
        if ($result instanceof PEAR_Error) {
            return $result;
        }

        return $this->_markNew();
    }

    /**
     * Move messages
     *
     * @param intiger $id message id
     * @param string $from Source folder
     * @param string $to Destination folder
     */
    public function moveMessage($id, $from = 'inbox', $to = 'archive')
    {
        $query = 'INSERT INTO usms_' . $to . ' (id, user_to, status, user_from, priority, title, chars, attaches) '
                . ' SELECT id, user_to, status, user_from, priority, title, chars, attaches FROM usms_' . $from
                . ' WHERE id IN (' . implode(',', $id) . ') AND user_to="'
                . Auth::getAuth() . '" ORDER BY id';
        $result = $this->_write_db->query($query);
        if ($result instanceof PEAR_Error) {
            return $result;
        }

        $query = 'DELETE FROM usms_' . $from . ' WHERE id IN (' . implode(',', $id)
                . ') AND user_to="' . Auth::getAuth() . '" ORDER BY id';
        $result = $this->_write_db->query($query);
        if ($result instanceof PEAR_Error) {
            return $result;
        }

        return $this->_markNew();
    }

    /**
     * Emptry trash
     *
     * @param string $folder Folder to clean up
     */
    public function emptyTrash($folder = 'inbox')
    {
        $query = 'SELECT id FROM usms_' . $folder . ' WHERE user_to="'
                . Auth::getAuth() . '" AND status="' . Usms::STATUS_DELETED . '" ORDER BY id';
        $ids = $this->_db->getCol($query);

        return $this->deleteMessage($ids, $folder, Auth::getAuth());
    }

    /**
     * Delete message
     *
     * @param intiger $id message id
     * @param string $folder folder name
     * @param string $user user owning the message
     */
    public function deleteMessage($id, $folder = 'inbox', $user = null)
    {
        if (empty($ids)) {
            return true;
        }

        if ($user === null) {
            $user = Auth::getAuth();
        }

        if (is_array($id)) {
            $query = 'DELETE FROM usms_' . $folder . ' WHERE id IN (' . implode(',', $id)
                    . ') AND user_to="' . $user . '"';
            $result = $this->_write_db->query($query);
        } else {
            $query = 'DELETE FROM usms_' . $folder
                   . ' WHERE id = ' . $id . ' AND user_to="' . $user .'"';
            $result = $this->_write_db->query($query);
        }

        if ($result instanceof PEAR_Error) {
            return $result;
        }

        // Delete files on the end
        register_shutdown_function(array(&$this, '_doGC'), $id, $user);

        return $this->_markNew();
    }

    /**
     * Delete sent message if was not read yet
     *
     * @param intiger $id message id
     * @param string $user user owning the message
     */
    public function deleteSentMessage($id, $user)
    {
        // set the DB portability to count deleted rows
        $this->_write_db->setOption('portability', DB_PORTABILITY_DELETE_COUNT);

        // try to delete the message if was not read yet
        $query = 'DELETE FROM usms_inbox WHERE '
                . ' id = ? AND status = ? AND user_to = ? AND user_from = ?';
        $params = array($id, Usms::STATUS_UNSEEN, $user, Auth::getAuth());
        $result = $this->_write_db->query($query, $params);
        if ($result instanceof PEAR_Error) {
            return $result;
        }

        if ($this->_write_db->affectedRows() > 0) {
            $this->_deleteSentMessage($id, $user);
            return $this->_markNew($user);
        } else {
            return PEAR::RaiseError(_("Message was already read."));
        }
    }

    /**
     * Mark users to have new data
     *
     * @param string $user user to mark
     */
    private function _markNew($user = null)
    {
        if ($user == null) {
            $user = Auth::getBareAuth();
        }

        if (is_array($user)) {
            $query = 'SELECT COUNT(*), user_to FROM usms_inbox WHERE user_to IN ("' . implode('","', $user).  '") '
                    . ' AND status = ' . Usms::STATUS_UNSEEN . ' GROUP BY user_to';
            $msgs = $this->_db->getAll($query);
            foreach ($msgs as $count) {
                $this->_writeNew($count[1], $count[0]);
            }
        } else {
            $query = 'SELECT COUNT(*) FROM usms_inbox WHERE user_to = ? AND status = ?';
            $msgs = $this->_db->getOne($query, array($user, Usms::STATUS_UNSEEN));
            $this->_writeNew($user, $msgs);
        }
    }

    /**
     * Check if a user exists
     *
     * @param string $user username to check
     */
    private function _userExists($user)
    {
        if (empty($GLOBALS['conf']['general']['exists'])) {
            return true;
        }

        $auth = &Auth::singleton($GLOBALS['conf']['auth']['driver']);
        return $auth->exists($user);
    }

    /**
     * Send message
     *
     * @param mixed $users username to send message to
     * @param array $info Message data
     * @param integer $notify Notify user that message was sent
     **/
    public function sendMessage($users, $info, $notify = true)
    {
        global $notification;

        // Any action ?
        if (empty($info['actionID'])) {
            $info['actionID'] = false;
        }

        // Add attachments
        if ($GLOBALS['conf']['general']['attaches']) {
            if ($info['actionID'] == 'forward') {
                $from_id = Util::getGet('id');
                $from_user = Util::getPost('user_from');
                $old = $this->readBodyData($from_id, $from_user);
                $info['attachments'] = $old['attachments'];
                $path = $this->vfsPath($from_user, $from_id);
                foreach ($info['attachments'] as $id => $file) {
                    $info['attachments'][$id]['file'] = $path . $id;
                }
            } else {
                for ($i = 0; $i < $GLOBALS['conf']['general']['attaches']; $i++) {
                    if (!(Browser::wasFileUploaded("attachment$i") instanceof PEAR_Error)) {
                        $info['attachments'][] = $info["attachment$i"];
                    }
                }
            }
        }

        // Prepare message data
        $index_data = array();
        if (is_array($users)) {
            $users = array_unique($users);
            // Check number of users
            if ($GLOBALS['conf']['general']['tolimit'] &&
                count($users) > $GLOBALS['conf']['general']['tolimit']) {
                $msg = sprintf(_("You cannot send a message to more then %d users at once."), $GLOBALS['conf']['general']['tolimit']);
                return PEAR::RaiseError($msg);
            }
        } else {
            $users = array($users);
        }

        $status = true;

        foreach ($users as $user) {

            $user = strtolower(trim($user));
            if (empty($user)) {
                continue;
            }

            // User exists?
            if (!$this->_userExists($user)) {
                $msg = sprintf(_("User '%s' don't exist."), $user);
                $status = PEAR::RaiseError($msg);
                if (sizeof($users) == 1) {
                    return $status;
                }
                $notification->push($msg, 'horde.error');
                continue;
            }

            // Are we on his blacklist?
            if (Usms::isBlacklisted($user)) {
                $msg = sprintf(_("You cannot send a message to %s since you are on his/her blacklist"), $user);
                $status = PEAR::RaiseError($msg);
                if (sizeof($users) == 1) {
                    return $status;
                }
                $notification->push($msg, 'horde.error');
                continue;
            }

            // Check title
            $info['title'] = strip_tags($info['title']);
            if (empty($info['title'])) {
                $info['title'] = _("No Subject");
            }

            // Create index
            $i = explode(' ', microtime());
            $index = $i[1] . substr($i[0], 2);
            $index_data[] = array($index,
                                  $user,
                                  Usms::STATUS_UNSEEN,
                                  Auth::getAuth(),
                                  isset($info['priority']) ? (int)$info['priority'] : Usms::PRIORITY_NORMAL,
                                  $info['title'],
                                  strlen($info['content']),
                                  isset($info['attachments']) ? sizeof($info['attachments']) : 0);

            if (!empty($info['attachments'])) {
                foreach ($info['attachments'] as $i => $file) {
                    if ($info['actionID'] == 'forward') {
                        $this->_copyFile($from_id, $from_user, $i, $index, $user);
                    } else {
                        $this->_storeFile($index, $file, $user);
                    }
                }
            }

            $this->_storeBodyData($index, $info, $user);
            if ($notify) {
                $notification->push(sprintf(_("Message to user '%s' was send successfuly."), $user), 'horde.success');
            }
        }

        $query = 'INSERT INTO usms_inbox (id, user_to, status, user_from, priority, title, chars, attaches) '
                . ' VALUES (?, ?, ?, ?, ?, ?, ?, ?)';
        $sth = $this->_write_db->prepare($query);
        $result = $this->_write_db->executeMultiple($sth, $index_data);
        if ($result instanceof PEAR_Error) {
            return $result;
        }

        $this->_markNew($users);

        if ($info['actionID'] == 'forward') {
            $this->changeStatus($from_id, 'inbox', Usms::STATUS_FORWARDED);
        } elseif ($info['actionID'] == 'replay') {
            $this->changeStatus(Util::getGet('id', 0), 'inbox', Usms::STATUS_REPLIED);
        } else {
            $this->_expireCache(Auth::getAuth());
        }

        return $status;
    }

    /**
     * Get cache value
     *
     * @param string $key Cache key
     */
    private function _getCache($key)
    {
        $key = $this->_getCacheKey($key);

        return $this->_cache->get($key, $GLOBALS['conf']['cache']['default_lifetime']);
    }

    /**
     * Set cache value
     *
     * @param string $key Cache key
     * @param string $value Cache value
     */
    private function _setCache($key, $value)
    {
        $key = $this->_getCacheKey($key);

        return $this->_cache->set($key, $value);
    }

    /**
     * Attempts to open a persistent connection to the SQL server.
     *
     * @return boolean  True on success; exits (Horde::fatal()) on error.
     */
    private function _connect()
    {
        Horde::assertDriverConfig($this->_params, 'storage',
                                  array('phptype', 'charset'));

        if (!isset($this->_params['database'])) {
            $this->_params['database'] = '';
        }
        if (!isset($this->_params['username'])) {
            $this->_params['username'] = '';
        }
        if (!isset($this->_params['hostspec'])) {
            $this->_params['hostspec'] = '';
        }
        if (isset($this->_params['prefix'])) {
            $this->prefix = $this->_params['prefix'];
        }

        /* Connect to the SQL server using the supplied parameters. */
        require_once 'DB.php';
        $this->_write_db = &DB::connect($this->_params,
                                        array('persistent' => !empty($this->_params['persistent'])));
        if ($this->_write_db instanceof PEAR_Error) {
            Horde::fatal($this->_write_db, __FILE__, __LINE__);
        }

        // Set DB portability options.
        switch ($this->_write_db->phptype) {
        case 'mssql':
            $this->_write_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
            break;
        default:
            $this->_write_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
        }

        /* Check if we need to set up the read DB connection seperately. */
        if (!empty($this->_params['splitread'])) {
            $params = array_merge($this->_params, $this->_params['read']);
            $this->_db = &DB::connect($params,
                                      array('persistent' => !empty($params['persistent'])));
            if ($this->_db instanceof PEAR_Error) {
                Horde::fatal($this->_db, __FILE__, __LINE__);
            }

            // Set DB portability options.
            switch ($this->_db->phptype) {
            case 'mssql':
                $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
                break;
            default:
                $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
            }

        } else {
            /* Default to the same DB handle for the writer too. */
            $this->_db =& $this->_write_db;
        }

        return true;
    }
}
