<?php
/**
 * USMS base class
 *
 * Copyright 2007 Obala d.o.o. (www.obala.si)
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://cvs.horde.org/co.php/news/LICENSE.
 *
 * $Id: Usms.php 311 2008-01-31 16:15:38Z duck $
 *
 * @author Duck <duck@obala.net>
 * @package Usms
 */
class Usms {

    const STATUS_SEEN = 0;
    const STATUS_UNSEEN = 2;
    const STATUS_REPLIED = 4;
    const STATUS_FORWARDED = 6;
    const STATUS_DELETED = 8;

    const PRIORITY_NORMAL = 0;
    const PRIORITY_LOW = 2;
    const PRIORITY_HIGH = 4;

    /**
     * Returns formatted time according to users prefs
     *
     * @param integer $time Unix timestamp
     */
    static function format_datetime($time)
    {
        return strftime($GLOBALS['prefs']->getValue('date_format'), $time)
            . ' '
            . (date($GLOBALS['prefs']->getValue('twentyFour') ? 'G:i' : 'g:ia', $time));
    }

    /**
     * Return message index or cooreponding timestamp
     *
     * @param int $index reversed microtime timestamp
     *
     * @return int index or unixtime
     */
    static public function index($index)
    {
        return substr($index, 0, 10);
    }

    /**
     * Format file size
     *
     * @param int $filesize filesize
     *
     * @return string formatted filesize.
     */
    public function format_filesize($filesize)
    {
        $units = array('B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB');

        $pass = 0; // set zero, for Bytes
        while ($filesize >= 1024) {
            $filesize /= 1024;
            $pass++;
        }

        return round($filesize, 2) . ' ' . $units[$pass];
    }

    /**
     * Add user to a friend list
     *
     * @param string $friend   Friend's usersame
     * @param string $group   Group to add friend to
     */
    static public function getGroups()
    {
        return array(_("Whitelist"));
    }

    /**
     * Add user to a friend list
     *
     * @param string $friend   Friend's usersame
     * @param string $group   Group to add friend to
     */
    static public function addFriend($friend, $group)
    {
        return _addrem_lists($group, $friend);
    }

    /**
     * Get user friends
     *
     * @return array of users
     */
    static public function getFriends()
    {
        return self::lists('whitelist');
    }

    /**
     * Get user blacklist
     *
     * @param string $user Username to check
     *
     * @return array of users blacklist
     */
    static public function getBlacklist($user)
    {
        return self::lists('blacklist', $user);
    }

    /**
     * Check if user is on blacklist
     *
     * @param string $black_for Blacklist owner
     * @param string $user    User to check
     *
     * @return boolean
     */
    static public function isBlacklisted($black_for, $user = null)
    {
        if ($user === null) {
            $user = Auth::getAuth();
        }

        if (Auth::isAdmin()
            || in_array($user, $GLOBALS['conf']['general']['blacklist'])
            || $user == Auth::getAuth()) {
            return false;
        }

        $blacklisted = self::getBlacklist($black_for);

        return array_key_exists($user, $blacklisted);
    }

    /**
     * Returns array of usernames in a list of false if list is empty
     * splits list by any number of commas or space characters
     * which include " ", \r, \t, \n and \f
     *
     * @param string $type List type to retreive
     * @param string $user Username to check
     *
     * @return array $list array fo usernames
     */
    static public function lists($type, $user = '')
    {
        if ($user == '' || $user == Auth::getAuth()) {
            $list = $GLOBALS['prefs']->getValue($type);
        } else {
            $userprefs = &Prefs::singleton($GLOBALS['conf']['prefs']['driver'], 'usms', $user);
            $list = $userprefs->getValue($type);
        }

        if ($list) {
            $users = preg_split("/[\s,]+/", $list, -1, PREG_SPLIT_NO_EMPTY);
            if (sizeof($users) > 0) {
                $list = array();
                foreach ($users as $value) {
                   $list[$value] = $value;
                }
                return $list;
            }
        }

        return false;
    }

    /**
     * add/remove a user from a list
     *
     * @param string $type of the list
     * @param string $user user to applay
     */
    static private function _addrem_lists($type, $user)
    {
        global $notification, $prefs;

        $list = $prefs->getValue($type);
        if ($list) {
            $users = preg_split("/[\s,]+/", $list, -1, PREG_SPLIT_NO_EMPTY);
            if (in_array($user, $users)) {
                $key = array_search($user, $users);
                unset($users[$key]);
                sort($users);
                $prefs->setValue($type, implode($users, ' '));
                $notification->push(sprintf(_("User '%s' removed"), $user) , 'horde.success');
            } else {
                $users[] = $user;
                sort($users);
                $prefs->setValue($type, implode($users, ' '));
                $notification->push(sprintf(_("User '%s' added"), $user) , 'horde.success');
            }
        } else {
            $prefs->setValue($type, $user);
            $notification->push(sprintf(_("User '%s' added"), $user) , 'horde.success');
        }

        return false;
    }

    /**
     * Build Usms's list of menu items
     */
    static public function getMenu($returnType = 'object')
    {
        global $registry;

        require_once 'Horde/Menu.php';

        $menu = &new Menu();
        $folder = Util::getGet('folder', 'inbox');
        $dir = $registry->getImageDir('horde');
        $url = Horde::applicationUrl('browse.php');
        $menu->add($url, _("Inbox"), 'mail.png', $dir, '', '', ($folder == 'inbox' ? 'current' : '__noselection'));
        $menu->add(Horde::applicationUrl('compose.php'), _("Compose"), 'edit.png', $dir, '', '', (basename($_SERVER['SCRIPT_NAME'] == 'compose.php') ? 'current' : '__noselection'));
        $menu->add(Util::addParameter($url, 'folder', 'sent'), _("Sent"), 'info.png', $dir, '', '', ($folder == 'sent' ? 'current' : '__noselection'));
        $menu->add(Util::addParameter($url, 'folder', 'archive'), _("Archive"), 'data.png', $dir, '', '', ($folder == 'archive' ? 'current' : '__noselection'));
        $menu->add(Util::addParameter($url, array('folder' => $folder, 'actionID' => 'empty_trash')), _("Empty trash"), 'empty_trash.png', null, '', '', '__noselection');

        $url = Horde::url($GLOBALS['registry']->get('webroot', 'horde') . '/services/prefs.php?group=lists&app=usms');

        $menu->add($url, _("Friends"), 'group.png', $dir);

        if ($returnType == 'object') {
            return $menu;
        } else {
            return $menu->render();
        }
    }

}
